/***************************************************************************
 *   Copyright (C) 2009 by Fabian Wuertz                                   *
 *   xadras@sidux.com                                                      *
 *   Project: Waheela                                                      *
 *   File:    waheela.cpp                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kapplication.h>
#include <klocale.h>
#include <KAboutData>
#include <KAboutApplicationDialog>
#include <KHelpMenu>
#include <kconfigskeleton.h>
#include <kconfigdialog.h>
#include <kstandarddirs.h>
#include <kglobal.h>
#include <KPasswordDialog>


#include "waheela.h"
#include "playercontroller.h"



//------------------------------------------------------------------------------
//--- init ---------------------------------------------------------------------
//------------------------------------------------------------------------------


Waheela::Waheela()
{



	// init gui
	setupUi(this);
	setWindowIcon(KIcon("waheela"));
	showFullScreen();


	// read config
	KConfig config("waheela");
	KConfigGroup generalGroup( &config, "General" );
	KConfigGroup amarokSettings( &config, "Amarok" );
	KConfigGroup dateSettings( &config, "Date" );
	KConfigGroup metadataSettings( &config, "Metadata" );
	KConfigGroup passwordSettings( &config, "Password" );

	// theme
	theme                 = generalGroup.readEntry( "Theme", "FloralGlow" );


	// date
	showDate              = dateSettings.readEntry( "ShowDate", true );
	showTime              = dateSettings.readEntry( "ShowTime", true );
	dateDefaultFont       = QFont("Sans Serif", 13, QFont::Normal);
	dateFont              = dateSettings.readEntry( "Font", dateDefaultFont );
	useCustomDateFont     = dateSettings.readEntry( "UseCustomFont", false );


	// metadata
	showCover             = metadataSettings.readEntry( "ShowCover", true );
	metadataDefaultFont   = QFont("Sans Serif", 16, QFont::Normal);
	metadataFont          = metadataSettings.readEntry( "metadataFont", metadataDefaultFont );
	useCustomMetadataFont = metadataSettings.readEntry( "useCustomMetadataFont", false );


	// password
	usePassword           = passwordSettings.readEntry( "UsePassword", false );
	password              = passwordSettings.readEntry( "Password", "" );
	
	//screen
	if(width()/height() > 1.3 )
		wideScreen = TRUE;


	// init amarok
	bool useCustomAmarokBinPath  = amarokSettings.readEntry("useCustomAmarokBinPath", false);
	QString amarokBinPath        = amarokSettings.readEntry("customAmarokBinPath", "/usr/bin/amarok");
	PlayerController::registerPlayWolfInstance();
	PlayerController::self()->setAmarokPath(useCustomAmarokBinPath, amarokBinPath);
	PlayerController::self()->updateInfos();

	// settings
	initSettingsDialog();

	// about
	initAboutDialog();


	// set up play buttons
	playButton->setIcon(KIcon("media-playback-start"));
	nextButton->setIcon(KIcon("media-skip-forward"));
	prevButton->setIcon(KIcon("media-skip-backward") );
	connect(playButton,SIGNAL(clicked()), this, SLOT(playSong()));   
	connect(prevButton,SIGNAL(clicked()), this, SLOT(prevSong()));
	connect(nextButton,SIGNAL(clicked()), this, SLOT(nextSong()));
	exitButton->setIcon(KIcon("application-exit"));
	aboutButton->setIcon(KIcon("waheela"));
	settingsButton->setIcon(KIcon("preferences-system"));
 	buttonWidget->setStyleSheet("border: 0");
 	bannerWidget->setStyleSheet("border: 0");
	connect(aboutButton,SIGNAL(clicked()), aboutDialog, SLOT(show()));
	hideButtons();
	timer = new QTimer(this);
	connect(timer, SIGNAL(timeout()), this, SLOT(hideButtons()));
	setMouseTracking(TRUE);




	// init first song
	QVariantMap metadata = PlayerController::self()->getMetadata();
	updateMetadata(metadata);
	volumeSlider->setValue(PlayerController::self()->volume());
	setStatus(PlayerController::self()->getStatus().int1);
	setPosition(PlayerController::self()->getPosition(), metadata.value("mtime").toInt());


	// update slots
	connect(PlayerController::self(), SIGNAL(metadataChanged(const QVariantMap &)), this,SLOT(updateMetadata(const QVariantMap &)));
	connect(PlayerController::self(), SIGNAL(statusChanged(const int &)), this, SLOT(setStatus(const int &)));
	connect(PlayerController::self(), SIGNAL(positionChanged(const int& , const int& )), this ,SLOT(setPosition(const int&, const int&)));
	connect(volumeSlider, SIGNAL(sliderMoved(const int&)), PlayerController::self(), SLOT(setVolume(const int&)));



	//clock
	updateClock();


}


//------------------------------------------------------------------------------
//--- clock ----------------------------------------------------------------------
//------------------------------------------------------------------------------

void Waheela::updateClock()
{
	QDateTime time = QDateTime::currentDateTime();
	dateLabel->setText(time.toString("dddd d. MMMM yyyy "));
	timeLabel->setText(time.toString(" h:mm"));
	QTimer::singleShot(10000, this, SLOT(updateClock()));
}

//------------------------------------------------------------------------------
//--- theme --------------------------------------------------------------------
//------------------------------------------------------------------------------

void Waheela::paintEvent(QPaintEvent *)
{

	QString themePath = KGlobal::dirs()->findResource ("data", "waheela/themes/"+theme+"/theme.config");
	KConfig config(themePath);


	// background image
	QString backgroundPath;
	if( wideScreen) {
		backgroundPath = KGlobal::dirs()->findResource ("data", "waheela/themes/"+theme+"/background-wide.jpg");
		if( backgroundPath.isEmpty() )
			backgroundPath = KGlobal::dirs()->findResource ("data", "waheela/themes/"+theme+"/background.jpg");
	}
	else {
		backgroundPath = KGlobal::dirs()->findResource ("data", "waheela/themes/"+theme+"/background.jpg");
		if( backgroundPath.isEmpty() )
			backgroundPath = KGlobal::dirs()->findResource ("data", "waheela/themes/"+theme+"/background-wide.jpg");
	}
	
	QPixmap pixmap( backgroundPath);
	QPainter painter(this);
	painter.drawPixmap(0, 0, this->width(), this->height(), pixmap);

	// info box
	KConfigGroup infoBoxThemeSettings( &config, "InfoBox" );
	QPoint infoBoxPosition = infoBoxThemeSettings.readEntry( "Position", QPoint(30,30) );
	infoBoxPosition.setX(displayWidget->width()/100*infoBoxPosition.x());
	infoBoxPosition.setY(displayWidget->height()/100*infoBoxPosition.y());
	infoBoxWidget->setGeometry(infoBoxPosition.x(), infoBoxPosition.y(),infoBoxWidget->width(),infoBoxWidget->height());
	QColor fontColor       = infoBoxThemeSettings.readEntry( "FontColor",  QColor( 255, 255, 255, 127) );
	metadataLabel->setPalette(QPalette(fontColor));
	if( useCustomMetadataFont )
		metadataLabel->setFont(metadataFont);
	else
		metadataLabel->setFont(metadataDefaultFont);

	// date & time
	KConfigGroup dateSettings( &config, "Date" );
	QColor backgroundColor = dateSettings.readEntry( "BackgroundColor", QColor(   0,   0, 255, 127) );
	fontColor              = dateSettings.readEntry( "FontColor",       QColor( 255, 255, 255, 127) );
	dateWidget->setPalette(QPalette(fontColor));
	if(showDate or showTime) {
		painter.setBrush(QColor(backgroundColor));
		painter.setPen(QColor(backgroundColor));
		painter.setOpacity(0.3);
		painter.drawRect(0, this->height()-55, this->width(),55);
		dateWidget->show();
		if( showDate)
			dateLabel->show();
		else
			dateLabel->hide();
		if( showTime)
			timeLabel->show();
		else
			timeLabel->hide();
	}
	else
		dateWidget->hide();
	if( useCustomDateFont )
		dateLabel->setFont(dateFont);
	else
		dateLabel->setFont(dateDefaultFont);
	QFont timeFont = dateLabel->font();
	timeFont.setBold(true);
	timeFont.setPointSize(dateLabel->font().pointSize()+3);
	timeLabel->setFont(timeFont);


	// progressBar
	KConfigGroup progressBarThemeSettings( &config, "ProgressBar" );
	QColor progressBarBackgroundColor = progressBarThemeSettings.readEntry( "BackgroundColor", QColor(255,255,255) );
	QColor progressBarForegroundColor = progressBarThemeSettings.readEntry( "ForegroundColor", QColor(103,141,178) );
	QPalette progressBarPalette(progressBarBackgroundColor);
	progressBarPalette.setColor(QPalette::Highlight, progressBarForegroundColor);
	progressBar->setPalette(progressBarPalette);

	// volumeSlider
	KConfigGroup volumeSliderThemeSettings( &config, "VolumeSlider" );
	QColor volumeSliderButtonColor = volumeSliderThemeSettings.readEntry( "ButtonColor", QColor(255,255,255) );
	QPalette volumeSliderPalette(QColor(255,255,255));
	volumeSliderPalette.setColor(QPalette::Button, volumeSliderButtonColor);
	volumeSlider->setPalette(volumeSliderPalette);

}



//------------------------------------------------------------------------------
//--- settings -----------------------------------------------------------------
//------------------------------------------------------------------------------


void Waheela::initSettingsDialog()
{
	KConfigSkeleton* test = new KConfigSkeleton;
	dialog = new KConfigDialog(this, "settings", test);
	connect(settingsButton,SIGNAL(clicked()), dialog, SLOT(show()));   
	connect(dialog, SIGNAL(okClicked()), this, SLOT(configAccepted()));

	// theme page
	QWidget *themesWidget = new QWidget();
	themesUi.setupUi(themesWidget);
	dialog->addPage(themesWidget, i18n("Themes"), "applications-graphics");
	dialog->setButtons( KDialog::Ok | KDialog::Cancel);

	QStringList themes = KGlobal::dirs()->findAllResources ("data", "waheela/themes/*/theme.config");
	themes.replaceInStrings("/theme.config", "");

	int i = 0;
	int j = 0;
	QString spacerPath = KGlobal::dirs()->findResource ("data", "waheela/spacer.png");
	foreach(QString themeName, themes) {
		QListWidgetItem *newItem = new QListWidgetItem;

		if( QFile::exists(themeName+"/preview.jpg") )
			newItem->setIcon(QIcon(themeName+"/preview.jpg"));
		else
			newItem->setIcon (QIcon(spacerPath));

		themeName = themeName.split("/").last();
		newItem->setText(themeName);
		if(themeName == theme)
			j = i;
		i++;
		themesUi.themesListWidget->addItem(newItem);
	}
	themesUi.themesListWidget->setCurrentRow(j);
	themesUi.themesListWidget->sortItems();



	// metadata page
	QWidget *metadataWidget = new QWidget();
	metadataUi.setupUi(metadataWidget);
	dialog->addPage(metadataWidget, i18n("Metadata"), "text-x-texinfo.png");
	metadataUi.showCoverCheckBox->setChecked( showCover );
    metadataUi.useCustomMetadataFontCheckBox->setChecked(useCustomMetadataFont);
	metadataUi.fontChooser->setEnabled(useCustomMetadataFont);
    metadataUi.fontChooser->setFont(metadataFont);


	// date page
	QWidget *dateWidget = new QWidget();
	dateUi.setupUi(dateWidget);
	dialog->addPage(dateWidget, i18n("Date&Time"), "xclock");
	dateUi.showDateCheckBox->setChecked( showDate );
	dateUi.showTimeCheckBox->setChecked( showTime );
    dateUi.useCustomFontCheckBox->setChecked(useCustomDateFont);
	dateUi.fontChooser->setEnabled(useCustomDateFont);
    dateUi.fontChooser->setFont(dateFont);

	// password page
	QWidget *passwordWidget = new QWidget();
	passwordUi.setupUi(passwordWidget);
	dialog->addPage(passwordWidget, i18n("Password"), "document-encrypt");
	passwordUi.usePasswordCheckBox->setChecked( usePassword );
	passwordUi.passwordLineEdit->setText( password );


	// amarok page
	QWidget *amarokWidget = new QWidget();
	amarokUi.setupUi(amarokWidget);
	dialog->addPage(amarokWidget, "Amarok", "amarok");
	amarokUi.customAmarokBinPath->setText(PlayerController::self()->amarokPath());
	amarokUi.customAmarokBinPath->setEnabled(PlayerController::self()->useCustomAmarokPath());
	amarokUi.useCustomAmarokBinPath->setChecked(PlayerController::self()->useCustomAmarokPath());

}


void Waheela::configAccepted()
{

	KConfig config("waheela");
	KConfigGroup generalGroup( &config, "General" );
	KConfigGroup amarokSettings( &config, "Amarok" );
	KConfigGroup dateSettings( &config, "Date" );
	KConfigGroup metadataSettings( &config, "Metadata" );
	KConfigGroup passwordSettings( &config, "Password" );

	// amarok page
	bool useCustom = amarokUi.useCustomAmarokBinPath->isChecked();
	amarokSettings.writeEntry("useCustomAmarokBinPath", useCustom);
	QString amarokBinPath = amarokUi.customAmarokBinPath->text();
	amarokSettings.writeEntry("customAmarokBinPath", amarokBinPath);

	// date page
	showDate  = dateUi.showDateCheckBox->isChecked();
	dateSettings.writeEntry( "ShowDate", showDate );
	showTime  = dateUi.showTimeCheckBox->isChecked();
	dateSettings.writeEntry( "ShowTime", showTime );
    useCustomDateFont = dateUi.useCustomFontCheckBox->isChecked();
    dateFont = dateUi.fontChooser->font();
    dateSettings.writeEntry("UseCustomFont", useCustomDateFont);
    dateSettings.writeEntry("Font", dateFont);


	// metadata page
	showCover = metadataUi.showCoverCheckBox->isChecked();
	metadataSettings.writeEntry( "ShowCover", showCover );
    useCustomMetadataFont = metadataUi.useCustomMetadataFontCheckBox->isChecked();
    metadataFont = metadataUi.fontChooser->font();
    metadataSettings.writeEntry("useCustomMetadataFont", useCustomMetadataFont);
    metadataSettings.writeEntry("metadataFont", metadataFont);

	// theme page
	theme = themesUi.themesListWidget->currentItem()->text();
	generalGroup.writeEntry( "Theme", theme );


	// theme page
	password = passwordUi.passwordLineEdit->text();
	passwordSettings.writeEntry( "Password", password );
	usePassword = passwordUi.usePasswordCheckBox->isChecked();
	passwordSettings.writeEntry( "UsePassword", usePassword );



	repaint();
	updateMetadata(PlayerController::self()->getMetadata());


}




//------------------------------------------------------------------------------
//--- button mode --------------------------------------------------------------
//------------------------------------------------------------------------------


void Waheela::playSong()
{
	PlayerController::self()->playPause();
}


void Waheela::stopSong()
{
	PlayerController::self()->stop();
}


void Waheela::prevSong()
{
	PlayerController::self()->prev();
}

void Waheela::nextSong()
{
	PlayerController::self()->next();
}


void Waheela::setPosition(const int pos, const int total) {
	progressBar->setValue(pos);
	progressBar->setMaximum(total);
}

void Waheela::setStatus(const int &status) {

	if (status == 1)
		playButton->setIcon(KIcon("media-playback-start"));
	else if (status == 0)
		playButton->setIcon(KIcon("media-playback-pause"));
	else
		playButton->setIcon(KIcon("media-playback-start"));
}


void Waheela::hideButtons()
{
	playButton->hide();
	nextButton->hide();
	prevButton->hide();
	exitButton->hide();
	aboutButton->hide();
	settingsButton->hide();
	volumeBox->hide();
	pwd = "";
}



void Waheela::updateMetadata(const QVariantMap& metadata) {

	metadataLabel->setText(metadata.value("title").toString()+"\n\n"+metadata.value("artist").toString()+"\n\n"+metadata.value("album").toString());

	int i = metadata.value("location").toString().indexOf(":");
	QString mode = metadata.value("location").toString().left(i);

	if( showCover ) {
		QString cover;
		if (metadata.value("arturl").toString() != QString()) {
			cover = metadata.value("arturl").toString();
			cover.replace("file://", "");
		}
		else if (mode == QString("lastfm"))
			cover = "/usr/share/kde4/apps/amarok/images/lastfm-default-cover.png";
		else
			cover = "/usr/share/kde4/apps/amarok/images/nocover.png";

		QPixmap coverPixmap = QPixmap(cover);
		if( ! coverPixmap.isNull() )
			coverPixmap = coverPixmap.scaledToWidth(200, Qt::SmoothTransformation);
		coverLabel->setPixmap(coverPixmap);
	}
 	else
		coverLabel->setPixmap(QPixmap());
}




//------------------------------------------------------------------------------
//--- mouse events -------------------------------------------------------------
//------------------------------------------------------------------------------



void Waheela::wheelEvent(QWheelEvent *event) {

	if( usePassword and pwd != password ) {
		KPasswordDialog *dlg = new KPasswordDialog( this );
		dlg->setPrompt(i18n("Enter a password"));
		connect( dlg, SIGNAL( gotPassword( const QString& , bool ) )  , this, SLOT( setPassword( const QString &) ) );
		dlg->show();
		return;
	}

	if (event->delta() > 0)
		PlayerController::self()->volumeUp();
	else
		PlayerController::self()->volumeDown();
	volumeSlider->setValue(PlayerController::self()->volume());

	timer->start(10000);
}


void Waheela::mousePressEvent(QMouseEvent *event)
{

	if( usePassword and pwd != password ) {
		KPasswordDialog *dlg = new KPasswordDialog( this );
		dlg->setPrompt(i18n("Enter a password"));
		connect( dlg, SIGNAL( gotPassword( const QString& , bool ) )  , this, SLOT( setPassword( const QString &) ) );
		dlg->show();
		return;
	}



	timer->start(10000);

	int cursorX = event->pos().x();
	int cursorY = event->pos().y();
	int progressBarX = progressBar->pos().x();
	int progressBarY = progressBar->pos().y();
	int progressBarHeight = progressBar->height();
	int progressBarWidth = progressBar->width();
	
	if ( cursorY > progressBarY and cursorY < progressBarY+progressBarHeight and cursorX > progressBarX and cursorX < progressBarX+progressBarWidth) {
		int value = (cursorX-progressBarX)*progressBar->maximum()/progressBarWidth+1;
		progressBar->setValue(value); 
		PlayerController::self()->seek(value);
	}
	else {
		playButton->show();
		nextButton->show();
		prevButton->show();
		exitButton->show();
		aboutButton->show();
		settingsButton->show();
		volumeBox->show();
	}


}


void Waheela::setPassword(QString input) {
	pwd = input;
}


//------------------------------------------------------------------------------
//--- about --------------------------------------------------------------------
//------------------------------------------------------------------------------

void Waheela::initAboutDialog()
{
	KAboutData *about = new KAboutData( 
		"Waheela",
		0,
		ki18n("Waheela"),
		KDE_VERSION_STRING,
		KLocalizedString(),
		KAboutData::License_GPL,
		ki18n("Copyright 2009 Fabian Wuertz")
	);
	about->addAuthor(ki18n("Fabian Wuertz"), KLocalizedString(), "xadras@sidux.com");
	about->addAuthor(ki18n("Giulio Camuffo"), KLocalizedString(), "");
	about->setProgramIconName("waheela");
	about->setProgramLogo(KIcon("waheela"));
	about->setVersion("0.2.1");
	about->setBugAddress("http://sidux.com");
 	aboutDialog = new KAboutApplicationDialog ( about, this );
}

